//============================================================================
// Name        : FreeMindOut.cpp
// Author      : Antonio Molina
// Version     :
// Copyright   : GPL 3.0
//============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sstream>
#include <math.h>

#include "Tablero.h"
#include "MenuJuego.h"
#include "definitions.h"

using namespace STDP;

void menuPrincipal(StdPijo &stdp, int seleccionado);
void modoUnlimited(StdPijo&);
void modoHistoria(StdPijo &stdp, int);
void modoSupervivencia(StdPijo &stdp);
void imprimirTextoCentrado(StdPijo &stdp, string texto, int fila, int margenIzq, Color colorTexto, Color colorFondo, Attribute atributo);
void imprimirHistoria(int capitulo, StdPijo &stdp);

const int numHistorias = 5; //Número de campañas en el modo historia

int main(void) {
	//Semilla random
	srand (time(NULL));

	StdPijo& stdp=StdPijo::p();
	stdp.init();
	stdp.setGlobalEscapeDelay(0);

	int numCapitulos=5;
	int pos=0;
	char got;
	bool continuar = true;

	while(continuar){
		stdp.clearScreen();
		menuPrincipal(stdp, pos);
		stdp.refreshScr();
		usleep(5000);

		fflush(stdin);
		got=stdp.getKey();
		switch(got){
			case 'w':if(pos>0)
					pos--;
				break;
			case 's':if(pos<3)
					pos++;
				break;
			case ' ':
				switch(pos){
					case 0: modoHistoria(stdp, numCapitulos);
						break;
					case 1: modoUnlimited(stdp);
						break;
					case 2: modoSupervivencia(stdp);
						break;
					case 3: continuar=false;
						break;
				}
				break;
		}


	}

	return EXIT_SUCCESS;
}

void modoUnlimited(StdPijo &stdp){
	//Texto explicativo del modo de juego
	int lineasDesc=6;
	string descripcion[lineasDesc];
	descripcion[0]="Este modo se basa";
	descripcion[1]="en un tablero dinámico";
	descripcion[2]="cuyo tamaño y ";
	descripcion[3]="densidad de minas crecerán";
	descripcion[4]="a medida que el";
	descripcion[5]="jugador suba de nivel.";

	struct timeval ti, tf;
	double tiempo;
	bool vivo = true;

	gettimeofday(&ti, NULL);   // Instante inicial

	Nivel nivel= Nivel();

	MenuJuego menuJuego = MenuJuego(stdp);
	Tablero tb = Tablero(stdp, nivel, CYELLOW, CWHITE, CYELLOW, '&');
	bool seguir = true;

	tb.sembrarMinas();

	while(seguir){
		do{
			stdp.clearScreen();
			menuJuego.setMargen(nivel.getColumnas()*2);

			gettimeofday(&tf, NULL);   // Instante final
			tiempo= (tf.tv_sec - ti.tv_sec)*1000 + (tf.tv_usec - ti.tv_usec)/1000.0;
			menuJuego.setTiempoJuego(tiempo/1000);
			menuJuego.setNivel(nivel.getNumNivel());

			tb.imprimirTablero(true);
//			tb.imprimirMinas(); //Quitando el comentario de la izquierda obtienes el poder


			//Movimiento del jugador
			fflush(stdin);
			char got = stdp.getKey();
			Posicion jug = tb.getJugador();
			switch(got){
				case 'd': jug.x+=2;
					vivo=tb.moverJugador(jug);
					break;
				case 'a': jug.x-=2;
					vivo=tb.moverJugador(jug);
					break;
				case 's': jug.y++;
					vivo=tb.moverJugador(jug);
					break;
				case 'w': jug.y--;
					vivo=tb.moverJugador(jug);
					break;
				case KESCAPE: seguir=false;
					break;
			}

			//ImprimirMenu
			menuJuego.setMargen(nivel.getColumnas());
			menuJuego.setMinas(tb.detectarMinas(tb.getJugador()));
			for(int i=0; i<lineasDesc; i++){
				menuJuego.imprimirTextoCentrado(descripcion[i], 10+i, CWHITE, ABLINK);
			}
			menuJuego.imprimirMenuJuegoStandard();

			//Imprimir nota de salida
			imprimirTextoCentrado(stdp, "Pulsa 'Esc' para salir de la partida en cualquier momento.", MAXFILA-1, 0, CYELLOW, CBLACK, ANORMAL);

			//Nivel completado
			if(tb.getJugador().y == MARGENSUP)
				seguir=false;


			stdp.refreshScr();
			usleep(30000);

		}while(seguir && vivo);

		if(tb.getJugador().y == MARGENSUP){
			seguir=true;

			//Subimos de nivel
			nivel.siguienteNivelStandard();
			nivel.sumarPuntos(((double)nivel.getNumNivel()/tiempo)*1000000);
			menuJuego.setPuntos(nivel.getPuntos());

			//Mostramos el tablero resuelto y damos la posibilidad de salir o cuntinuar
			while(seguir){
				stdp.clearScreen();
				tb.imprimirTablero(true);
				menuJuego.mostrarFinalPartidaStandard();
				tb.imprimirMinas();

				char got = stdp.getKey();

				if(got==' '){
					tb.sembrarMinas();
					break;
				}
				if(got==KESCAPE)
					seguir=false;

				stdp.refreshScr();
				usleep(5000);
			}
		}
		if(!vivo){
			//if(nivel.getPuntos()>0)
				nivel.sumarPuntos(-100);
			seguir=true;
			vivo=true;
			while(seguir){
				stdp.clearScreen();
				tb.imprimirTablero(true);
				menuJuego.mostrarFalloStandard();
				tb.imprimirMinas();

				char got = stdp.getKey();

				if(got==' '){
					tb.sembrarMinas();
					break;
				}
				if(got==KESCAPE)
					seguir=false;

				stdp.refreshScr();
				usleep(5000);
			}
		}
		gettimeofday(&ti, NULL);//Reiniciamos el cronómetro
	}
}

void modoHistoria(StdPijo &stdp, int numCapitulos){

	//Texto explicativo del modo de juego
	int lineasDesc=6;
	string descripcion[numHistorias][lineasDesc];
	descripcion[0][0]="Debes atravesar la frontera.";
	descripcion[0][1]="Has hackeado los servidores";
	descripcion[0][2]="de la C.I.A. y sabes que";
	descripcion[0][3]="hay una docena de minas en";
	descripcion[0][4]="este fragmento.";
	descripcion[0][5]="Buena suerte.";

	descripcion[1][0]="Tienes ante ti el canal";
	descripcion[1][1]="de Panamá.";
	descripcion[1][2]="Cientos de personas murieron";
	descripcion[1][3]="En su construcción,";
	descripcion[1][4]="confío que no corras la";
	descripcion[1][5]="misma suerte.";

	descripcion[2][0]="Ante ti se abre el Atlántico.";
	descripcion[2][1]="Este escenario es grande y";
	descripcion[2][2]="peligroso. Tendrás que";
	descripcion[2][3]="poner toda la carne en el";
	descripcion[2][4]="asador y confiar que no se";
	descripcion[2][5]="desvanezca el campo magnético.";

	descripcion[3][0]="En este pequeño escenario";
	descripcion[3][1]="tendrás que darlo todo.";
	descripcion[3][2]="No dispondrás de rastro";
	descripcion[3][3]="y los bidones son cuantiosos.";
	descripcion[3][4]="";
	descripcion[3][5]="Si no los hubiesemos vertido...";

	Color lineasTablero;
	Color bordesTablero;
	Color colorCursor;

	struct timeval ti, tf;
	double tiempo;
	bool vivo = true;
	bool seguir = true;
	bool seguirGlobal=true;
	int margenSupTab;
	int margenIzqTab;
	bool rastro=true;

	Nivel nivel= Nivel();
	nivel.setNumNivel(1);
	MenuJuego menuJuego = MenuJuego(stdp);

	if(nivel.getNumNivel()==numCapitulos){
		imprimirHistoria(numCapitulos-1, stdp);
		seguirGlobal=false;
	}

	while(nivel.getNumNivel()<=numCapitulos && seguirGlobal){
		seguir=true;
		imprimirHistoria(nivel.getNumNivel()-1, stdp);

		gettimeofday(&ti, NULL);   // Instante inicial

		//Establecemos los parámetros de cada capítulo. Desde luego estaría mucho mejor que lo cargase dinámicamente desde un XML o algo así, pero...
		switch(nivel.getNumNivel()-1){
			case 0: lineasTablero=CWHITE;
				bordesTablero=CBLUE;
				colorCursor=CWHITE;
				nivel.setColumnas(40);
				nivel.setFilas(12);
				nivel.setNumMinas(10);
				margenSupTab=6;
				margenIzqTab=2;
				break;
			case 1: lineasTablero=CBLUE;
				bordesTablero=CMAGENTA;
				colorCursor=CWHITE;
				nivel.setColumnas(40);
				nivel.setFilas(11);
				nivel.setNumMinas(20);
				margenSupTab=6;
				margenIzqTab=2;
				break;
			case 2: lineasTablero=CBLUE;
				bordesTablero=CYELLOW;
				colorCursor=CWHITE;
				nivel.setColumnas(40);
				nivel.setFilas(16);
				nivel.setNumMinas(30);
				margenSupTab=3;
				margenIzqTab=2;
				break;
			case 3: lineasTablero=CYELLOW;
				bordesTablero=CBLACK;
				colorCursor=CWHITE;
				nivel.setColumnas(40);
				nivel.setFilas(11);
				nivel.setNumMinas(30);
				margenSupTab=4;
				margenIzqTab=2;
				rastro=false;
				break;
		}

		Tablero tb = Tablero(stdp, nivel, bordesTablero, lineasTablero, colorCursor, '&');
		tb.setMargenIzq(margenIzqTab);
		tb.setMargenSup(margenSupTab);

		tb.sembrarMinas();

		//Bucle del juego
		while(seguir){
			do{
					stdp.clearScreen();
					menuJuego.setMargen(nivel.getColumnas());

					//Obtenemos los parámetros del juego
					gettimeofday(&tf, NULL);   // Instante final
					tiempo= (tf.tv_sec - ti.tv_sec)*1000 + (tf.tv_usec - ti.tv_usec)/1000.0;
					menuJuego.setTiempoJuego(tiempo/1000);
					menuJuego.setNivel(nivel.getNumNivel());
					menuJuego.setMinas(tb.detectarMinas(tb.getJugador()));

					tb.imprimirTablero(rastro);
//					tb.imprimirMinas(); //Quitando el comentario de la izquierda obtienes el poder

					for(int i=0; i<lineasDesc; i++){
						menuJuego.imprimirTextoCentrado(descripcion[nivel.getNumNivel()-1][i], 10+i, CWHITE, ABLINK);
					}
					menuJuego.imprimirMenuJuegoHistoria(bordesTablero);
					//Imprimir nota de salida
					imprimirTextoCentrado(stdp, "Pulsa 'Esc' para salir de la partida en cualquier momento.", MAXFILA-1, 0, CYELLOW, CBLACK, ANORMAL);

					//Movimiento del jugador
					fflush(stdin);
					char got = stdp.getKey();
					Posicion jug = tb.getJugador();
					switch(got){
						case 'd': jug.x+=2;
							vivo=tb.moverJugador(jug);
							break;
						case 'a': jug.x-=2;
							vivo=tb.moverJugador(jug);
							break;
						case 's': jug.y++;
							vivo=tb.moverJugador(jug);
							break;
						case 'w': jug.y--;
							vivo=tb.moverJugador(jug);
							break;
						case KESCAPE: seguir=seguirGlobal=false;
							break;
					}

					//Nivel completado
					if(tb.getJugador().y == margenSupTab)
						seguir=false;

					stdp.refreshScr();
					usleep(30000);

			}while(seguir && vivo);

			if(tb.getJugador().y == margenSupTab){

				//Pasamos al siguiente capítulo
				nivel.incrementarNivel();

				gettimeofday(&ti, NULL);//Reiniciamos el cronómetro

				//Mostramos el tablero resuelto y damos la posibilidad de salir o cuntinuar
				while(seguir){
					stdp.clearScreen();
					tb.imprimirTablero(true);
					menuJuego.mostrarFinalPartidaHistoria(bordesTablero);
					tb.imprimirMinas();

					char got = stdp.getKey();

					if(got==' '){
						tb.sembrarMinas();
						break;
					}
					if(got==KESCAPE)
						seguir=false;

					stdp.refreshScr();
					usleep(5000);
				}
			}
			if(!vivo){
				seguir=true;
				vivo=true;
				while(seguir){
					stdp.clearScreen();
					tb.imprimirTablero(true);
					menuJuego.mostrarFalloHistoria(bordesTablero);
					tb.imprimirMinas();

					char got = stdp.getKey();

					if(got==' '){
						tb.sembrarMinas();
						break;
					}
					if(got==KESCAPE)
						seguir=false;

					stdp.refreshScr();
					usleep(5000);
				}
			}
			gettimeofday(&ti, NULL);//Reiniciamos el cronómetro
		}
	}
}

void modoSupervivencia(StdPijo &stdp){
	//Texto explicativo del modo de juego
	int lineasDesc=6;
	string descripcion[lineasDesc];
	descripcion[0]="Este modo se basa";
	descripcion[1]="en un número estático";
	descripcion[2]="de minas mientras que";
	descripcion[3]="el tablero es cada vez";
	descripcion[4]="más pequeño. Aquí no hay";
	descripcion[5]="puntos, solo supervivencia";

	struct timeval ti, tf;
	double tiempo;
	bool vivo = true;
	int filas=18;
	int colum=40;

	gettimeofday(&ti, NULL);   // Instante inicial

	Nivel nivel= Nivel();
	nivel.setNumMinas(30);
	nivel.setColumnas(colum);
	nivel.setFilas(filas);

	MenuJuego menuJuego = MenuJuego(stdp);
	Tablero tb = Tablero(stdp, nivel, CYELLOW, CWHITE, CYELLOW, '&');
	bool seguir = true;

	tb.sembrarMinas();

	while(seguir){
		do{
			stdp.clearScreen();
			menuJuego.setMargen(nivel.getColumnas()*2);

			gettimeofday(&tf, NULL);   // Instante final
			tiempo= (tf.tv_sec - ti.tv_sec)*1000 + (tf.tv_usec - ti.tv_usec)/1000.0;
			menuJuego.setTiempoJuego(tiempo/1000);
			menuJuego.setNivel(nivel.getNumNivel());

			tb.imprimirTablero(true);
//			tb.imprimirMinas(); //Quitando el comentario de la izquierda obtienes el poder


			//Movimiento del jugador
			fflush(stdin);
			char got = stdp.getKey();
			Posicion jug = tb.getJugador();
			switch(got){
				case 'd': jug.x+=2;
					vivo=tb.moverJugador(jug);
					break;
				case 'a': jug.x-=2;
					vivo=tb.moverJugador(jug);
					break;
				case 's': jug.y++;
					vivo=tb.moverJugador(jug);
					break;
				case 'w': jug.y--;
					vivo=tb.moverJugador(jug);
					break;
				case KESCAPE: seguir=false;
					break;
			}

			//ImprimirMenu
			menuJuego.setMargen(nivel.getColumnas());
			menuJuego.setMinas(tb.detectarMinas(tb.getJugador()));
			for(int i=0; i<lineasDesc; i++){
				menuJuego.imprimirTextoCentrado(descripcion[i], 10+i, CWHITE, ABLINK);
			}
			menuJuego.imprimirMenuJuegoSupervivencia();

			//Imprimir nota de salida
			imprimirTextoCentrado(stdp, "Pulsa 'Esc' para salir de la partida en cualquier momento.", MAXFILA-1, 0, CYELLOW, CBLACK, ANORMAL);

			//Nivel completado
			if(tb.getJugador().y == MARGENSUP)
				seguir=false;


			stdp.refreshScr();
			usleep(30000);

		}while(seguir && vivo);

		if(tb.getJugador().y == MARGENSUP){
			seguir=true;

			//Mostramos el tablero resuelto y damos la posibilidad de salir o cuntinuar
			while(seguir){
				stdp.clearScreen();
				tb.imprimirTablero(true);
				menuJuego.mostrarFinalPartidaSupervivencia();
				tb.imprimirMinas();

				char got = stdp.getKey();

				if(got==' '){
					tb.sembrarMinas();
					break;
				}
				if(got==KESCAPE)
					seguir=false;

				stdp.refreshScr();
				usleep(5000);
			}
			//Subimos de nivel
			if(nivel.getColumnas()>3)
				nivel.setColumnas(colum-=2);
		}
		if(!vivo){
			seguir=true;
			vivo=true;
			while(seguir){
				stdp.clearScreen();
				tb.imprimirTablero(true);
				menuJuego.mostrarFalloSupervivencia();
				tb.imprimirMinas();

				char got = stdp.getKey();

				if(got==' '){
					tb.sembrarMinas();
					break;
				}
				if(got==KESCAPE)
					seguir=false;

				stdp.refreshScr();
				usleep(5000);
			}
		}
		gettimeofday(&ti, NULL);//Reiniciamos el cronómetro
	}
}

void menuPrincipal(StdPijo &stdp, int seleccionado){
	int fila = 5;

	//Colores posibles
	Color textoNormal = CWHITE;
	Color fondoNormal = CBLACK;
	Color fondoSeleccionado = CWHITE;
	Color textoSeleccionado = CBLACK;

	string modos[4];
	modos[0]="Campaña";
	modos[1]="Unlimited";
	modos[2]="Supervivencia";
	modos[3]="Salir";

	imprimirTextoCentrado(stdp, "FreeMindOut", 2, 0, CGREEN, CBLACK, ABOLD);

	for(int i=0; i<4; i++){
		if(i==seleccionado)
			imprimirTextoCentrado(stdp, modos[i], fila, 0, textoSeleccionado, fondoSeleccionado, ABOLD);
		else
			imprimirTextoCentrado(stdp, modos[i], fila, 0, textoNormal, fondoNormal, ABOLD);
		fila++;
	}
	fila+=2;

	imprimirTextoCentrado(stdp, "Para moverse utilice las teclas 'w' y 's'.", fila++, 0, CYELLOW, fondoNormal, ANORMAL);
	imprimirTextoCentrado(stdp, "Pulse espacio para seleccionar.", fila++, 0, CYELLOW, fondoNormal, ANORMAL);
}

void imprimirTextoCentrado(StdPijo &stdp, string texto, int fila, int margenIzq, Color colorTexto, Color colorFondo, Attribute atributo){
	int espacio = 78;
	espacio -= texto.length();
	espacio /= 2;
	stdp.setCursor(espacio+margenIzq, fila);

	stdp.setColor(atributo, colorTexto, colorFondo);
	stdp.writeStr(texto.c_str());
}

void imprimirHistoria(int capitulo, StdPijo &stdp){
	char got;
	double espera=0.5*pow(10,5);
	stringstream scapitulo;
	if(capitulo==4)
		scapitulo << "Esto es todo amigos.";
	else
		scapitulo << "Capítulo " << capitulo+1 << ":";
	string historias[5][10];

	historias[0][0] = "En un congreso en el que tomaste dos copas de más,";
	historias[0][1] = "sin saber muy bien por qué, ";
	historias[0][2] = "enseñaste tu trasero al filantrópico Bill Gates.";
	historias[0][3] = "Ahora crees que la C.I.A. está detrás tuya.";
	historias[0][4] = "Aquí comienza tu gran aventura.";
	historias[0][5] = "...";
	historias[0][6] = "Para empezar decides huir de Estados Unidos";
	historias[0][7] = "por su frontera con México. Pero ten cuidado, estos americanos";
	historias[0][8] = "han minado la frontera para protegerse del sur";
	historias[0][9] = "y su constante oferta de mano de obra a explotar.";

	historias[1][0] = "Así que has conseguido cruzar la frontera...";
	historias[1][1] = "parece que es más fácil bajar que subir...";
	historias[1][2] = "De cualquier forma, no puedes quedarte aquí.";
	historias[1][3] = "Quizá sea buena idea cruzar el canal de Panamá";
	historias[1][4] = "Y buscar un buen lugar dónde esconderte.";
	historias[1][5] = "...";
	historias[1][6] = "Por desgracia, una nave de la marina estadounidense";
	historias[1][7] = "acaba de cruzar el canal y, aunque no te han visto,";
	historias[1][8] = "ha derramado cuantiosos explosivos a lo largo de este tramo.";
	historias[1][9] = "LLeva cuidado, nos vemos al otro lado.";

	historias[2][0] = "WikiLeaks ha filtrado unas imágenes tuyas posando en una";
	historias[2][1] = "playa jamaicana, parece que te tienen localizado.";
	historias[2][2] = "Será mejor que cruces el Atlántico y empieces una nueva";
	historias[2][3] = "vida en algún país lleno de oportunidades.";
	historias[2][4] = "Mientras tanto, procura no morir de inanición en uno de";
	historias[2][5] = "esos países tercermundistas.";
	historias[2][6] = "...";
	historias[2][7] = "Aunque pueda parecer inofensivo, el Atlántico está plagado de";
	historias[2][8] = "trampas colocadas por piratas somalies. No metas la pata y";
	historias[2][9] = "navega hasta la otra orilla.";

	historias[3][0] = "Sin saber muy bien como, has llegado hasta las playas de Somalia.";
	historias[3][1] = "Levantas la cabeza y adviertes que la costa está repleta de";
	historias[3][2] = "bidones radioactivos vertidos de forma completamente irregular.";
	historias[3][3] = "Más vale que no topes con ninguno de ellos o podría ser mortal.";
	historias[3][4] = "Para más inri, se está levantando una tormenta de arena y no puedes";
	historias[3][5] = "ver ni tus propias pisadas. Que Alá nos pille confesaos.";
	historias[3][6] = "...";
	historias[3][7] = "Este escenario, al ser el último, es el más difícil.";
	historias[3][8] = "Los bidones radioactivos son numerosos y no puedes";
	historias[3][9] = "ver el rastro que vas dejando.";

	historias[4][0] = "Pues esto es todo lo que me ha dado tiempo hacer.";
	historias[4][1] = "Espero, de alguna manera, haberos arrancado una sonrisa";
	historias[4][2] = "con esta historia inspirada (puntualmente) en";
	historias[4][3] = "escenarios reales.";
	historias[4][4] = "...";
	historias[4][5] = "Antonio Molina García-Retamero.";
	historias[4][6] = "";
	historias[4][7] = "";
	historias[4][8] = "";
	historias[4][9] = "";

	stdp.clearScreen();
	imprimirTextoCentrado(stdp, scapitulo.str(), 3, 0, CBLACK, CWHITE, ABOLD);
	imprimirTextoCentrado(stdp, "Pulse espacio para imprimir rápido.", 17, 0, CYELLOW, CBLACK, ANORMAL);

	for(int j=0; j<10; j++){
		for(int k=0; k<historias[capitulo][j].length()+1; k++){
			string texto= historias[capitulo][j].substr(0,k);
			imprimirTextoCentrado(stdp, texto, 5+j, 0, CWHITE, CBLACK, ANORMAL);
			stdp.refreshScr();
			if(got=stdp.getKey()==' ')
				espera=0;
			usleep(espera);
		}
	}

	imprimirTextoCentrado(stdp, "Pulse espacio para continuar.", 18, 0, CYELLOW, CBLACK, ANORMAL);


	while(got != ' '){
		fflush(stdin);
		got=stdp.getKey();
		usleep(300000);
	}


}
